@@ -51,6 +51,16 @@ class FlavorsTest(utils.TestCase):
         self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
         self.cs.assert_called('GET', '/flavors/detail?limit=4&marker=1234')
 
+    def test_list_flavors_with_min_disk(self):
+        fl = self.cs.flavors.list(min_disk=20)
+        self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
+        self.cs.assert_called('GET', '/flavors/detail?minDisk=20')
+
+    def test_list_flavors_with_min_ram(self):
+        fl = self.cs.flavors.list(min_ram=512)
+        self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
+        self.cs.assert_called('GET', '/flavors/detail?minRam=512')
+
     def test_list_flavors_with_sort_key_dir(self):
         fl = self.cs.flavors.list(sort_key='id', sort_dir='asc')
         self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
@@ -121,7 +131,8 @@ class FlavorsTest(utils.TestCase):
         self.assertRaises(exceptions.NotFound, self.cs.flavors.find,
                           disk=12345)
 
-    def _create_body(self, name, ram, vcpus, disk, ephemeral, id, swap,
+    @staticmethod
+    def _create_body(name, ram, vcpus, disk, ephemeral, id, swap,
                      rxtx_factor, is_public):
         return {
             "flavor": {
@@ -248,3 +259,65 @@ class FlavorsTest(utils.TestCase):
             mock.call("/flavors/1/os-extra_specs/k1"),
             mock.call("/flavors/1/os-extra_specs/k2")
         ])
+
+
+class FlavorsTest_v2_55(utils.TestCase):
+    """Tests creating/showing/updating a flavor with a description."""
+    def setUp(self):
+        super(FlavorsTest_v2_55, self).setUp()
+        self.cs = fakes.FakeClient(api_versions.APIVersion('2.55'))
+
+    def test_list_flavors(self):
+        fl = self.cs.flavors.list()
+        self.cs.assert_called('GET', '/flavors/detail')
+        for flavor in fl:
+            self.assertTrue(hasattr(flavor, 'description'),
+                            "%s does not have a description set." % flavor)
+
+    def test_list_flavors_undetailed(self):
+        fl = self.cs.flavors.list(detailed=False)
+        self.cs.assert_called('GET', '/flavors')
+        for flavor in fl:
+            self.assertTrue(hasattr(flavor, 'description'),
+                            "%s does not have a description set." % flavor)
+
+    def test_get_flavor_details(self):
+        f = self.cs.flavors.get('with-description')
+        self.cs.assert_called('GET', '/flavors/with-description')
+        self.assertEqual('test description', f.description)
+
+    def test_create(self):
+        self.cs.flavors.create(
+            'with-description', 512, 1, 10, 'with-description', ephemeral=10,
+            is_public=False, description='test description')
+
+        body = FlavorsTest._create_body(
+            "with-description", 512, 1, 10, 10, 'with-description',
+            0, 1.0, False)
+        body['flavor']['description'] = 'test description'
+        self.cs.assert_called('POST', '/flavors', body)
+
+    def test_create_bad_version(self):
+        """Tests trying to create a flavor with a description before 2.55."""
+        self.cs.api_version = api_versions.APIVersion('2.54')
+        self.assertRaises(exceptions.UnsupportedAttribute,
+                          self.cs.flavors.create,
+                          'with-description', 512, 1, 10, 'with-description',
+                          description='test description')
+
+    def test_update(self):
+        updated_flavor = self.cs.flavors.update(
+            'with-description', 'new description')
+        body = {
+            'flavor': {
+                'description': 'new description'
+            }
+        }
+        self.cs.assert_called('PUT', '/flavors/with-description', body)
+        self.assertEqual('new description', updated_flavor.description)
+
+    def test_update_bad_version(self):
+        """Tests trying to update a flavor with a description before 2.55."""
+        self.cs.api_version = api_versions.APIVersion('2.54')
+        self.assertRaises(exceptions.VersionNotFoundForAPIMethod,
+                          self.cs.flavors.update, 'foo', 'bar')
