@@ -47,44 +47,39 @@ class ServersTest(utils.FixturedTestCase):
         """
         return None
 
-    def test_list_all_servers(self):
+    def test_list_servers(self):
         sl = self.cs.servers.list()
         self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('GET', '/servers/detail', pos=-2)
-        self.assert_called('GET', '/servers/detail?marker=9012')
+        self.assert_called('GET', '/servers/detail')
         for s in sl:
             self.assertIsInstance(s, servers.Server)
 
     def test_filter_servers_unicode(self):
         sl = self.cs.servers.list(search_opts={'name': u't€sting'})
         self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called(
-            'GET',
-            '/servers/detail?name=t%E2%82%ACsting',
-            pos=-2)
-        self.assert_called(
-            'GET',
-            '/servers/detail?marker=9012&name=t%E2%82%ACsting')
+        self.assert_called('GET', '/servers/detail?name=t%E2%82%ACsting')
+        for s in sl:
+            self.assertIsInstance(s, servers.Server)
+
+    def test_list_all_servers(self):
+        # use marker just to identify this call in fixtures
+        sl = self.cs.servers.list(limit=-1, marker=1234)
+        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
+
+        self.assertEqual(2, len(sl))
+
+        self.assertEqual(self.requests_mock.request_history[-2].method, 'GET')
+        self.assertEqual(self.requests_mock.request_history[-2].path_url,
+                         '/servers/detail?marker=1234')
+        self.assert_called('GET', '/servers/detail?marker=5678')
+
         for s in sl:
             self.assertIsInstance(s, servers.Server)
 
     def test_list_servers_undetailed(self):
         sl = self.cs.servers.list(detailed=False)
         self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('GET', '/servers', pos=-2)
-        self.assert_called('GET', '/servers?marker=5678')
-        for s in sl:
-            self.assertIsInstance(s, servers.Server)
-
-    def test_list_servers_with_marker(self):
-        sl = self.cs.servers.list(marker=1234)
-        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
-
-        self.assertEqual(2, len(sl))
-
-        self.assert_called('GET', '/servers/detail?marker=1234', pos=-2)
-        self.assert_called('GET', '/servers/detail?marker=9012')
-
+        self.assert_called('GET', '/servers')
         for s in sl:
             self.assertIsInstance(s, servers.Server)
 
@@ -94,17 +89,6 @@ class ServersTest(utils.FixturedTestCase):
         self.assert_called('GET', '/servers/detail?limit=2&marker=1234')
         for s in sl:
             self.assertIsInstance(s, servers.Server)
-        self.assertEqual(2, len(sl))
-
-    def test_list_servers_with_limit_above_max_limit(self):
-        # use limit=3 to trigger paging simulation on backend fixture side
-        sl = self.cs.servers.list(limit=3)
-        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('GET', '/servers/detail?limit=3', pos=-2)
-        self.assert_called('GET', '/servers/detail?limit=1&marker=5678')
-        for s in sl:
-            self.assertIsInstance(s, servers.Server)
-        self.assertEqual(3, len(sl))
 
     def test_list_servers_sort_single(self):
         sl = self.cs.servers.list(sort_keys=['display_name'],
@@ -112,10 +96,7 @@ class ServersTest(utils.FixturedTestCase):
         self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called(
             'GET',
-            '/servers/detail?sort_dir=asc&sort_key=display_name', pos=-2)
-        self.assert_called(
-            'GET',
-            '/servers/detail?marker=9012&sort_dir=asc&sort_key=display_name')
+            '/servers/detail?sort_dir=asc&sort_key=display_name')
         for s in sl:
             self.assertIsInstance(s, servers.Server)
 
@@ -126,11 +107,6 @@ class ServersTest(utils.FixturedTestCase):
         self.assert_called(
             'GET',
             ('/servers/detail?sort_dir=asc&sort_dir=desc&'
-             'sort_key=display_name&sort_key=id'),
-            pos=-2)
-        self.assert_called(
-            'GET',
-            ('/servers/detail?marker=9012&sort_dir=asc&sort_dir=desc&'
              'sort_key=display_name&sort_key=id'))
         for s in sl:
             self.assertIsInstance(s, servers.Server)
@@ -1566,3 +1542,115 @@ class ServersV257Test(ServersV256Test):
             exceptions.UnsupportedAttribute, s.rebuild, image=1, name='new',
             meta={'foo': 'bar'}, files=files)
         self.assertIn('files', six.text_type(ex))
+
+
+class ServersV263Test(ServersV257Test):
+
+    api_version = "2.63"
+
+    def test_create_server_with_trusted_image_certificates(self):
+        self.cs.servers.create(
+            name="My server",
+            image=1,
+            flavor=1,
+            meta={'foo': 'bar'},
+            userdata="hello moto",
+            key_name="fakekey",
+            nics=self._get_server_create_default_nics(),
+            trusted_image_certificates=['id1', 'id2'],
+        )
+        self.assert_called('POST', '/servers',
+                           {'server': {
+                               'flavorRef': '1',
+                               'imageRef': '1',
+                               'key_name': 'fakekey',
+                               'max_count': 1,
+                               'metadata': {'foo': 'bar'},
+                               'min_count': 1,
+                               'name': 'My server',
+                               'networks': 'auto',
+                               'trusted_image_certificates': ['id1', 'id2'],
+                               'user_data': 'aGVsbG8gbW90bw=='
+                           }}
+                           )
+
+    def test_create_server_with_trusted_image_certificates_pre_263_fails(self):
+        self.cs.api_version = api_versions.APIVersion('2.62')
+        ex = self.assertRaises(
+            exceptions.UnsupportedAttribute, self.cs.servers.create,
+            name="My server", image=1, flavor=1, meta={'foo': 'bar'},
+            userdata="hello moto", key_name="fakekey",
+            nics=self._get_server_create_default_nics(),
+            trusted_image_certificates=['id1', 'id2'])
+        self.assertIn('trusted_image_certificates', six.text_type(ex))
+
+    def test_rebuild_server_with_trusted_image_certificates(self):
+        s = self.cs.servers.get(1234)
+        ret = s.rebuild(image="1", trusted_image_certificates=['id1', 'id2'])
+        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action',
+                           {'rebuild': {
+                               'imageRef': '1',
+                               'trusted_image_certificates': ['id1', 'id2']}})
+
+    def test_rebuild_server_with_trusted_image_certificates_none(self):
+        s = self.cs.servers.get(1234)
+        ret = s.rebuild(image="1", trusted_image_certificates=None)
+        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action',
+                           {'rebuild': {
+                               'imageRef': '1',
+                               'trusted_image_certificates': None}})
+
+    def test_rebuild_with_trusted_image_certificates_pre_263_fails(self):
+        self.cs.api_version = api_versions.APIVersion('2.62')
+        ex = self.assertRaises(exceptions.UnsupportedAttribute,
+                               self.cs.servers.rebuild,
+                               '1234', fakes.FAKE_IMAGE_UUID_1,
+                               trusted_image_certificates=['id1', 'id2'])
+        self.assertIn('trusted_image_certificates', six.text_type(ex))
+
+
+class ServersV267Test(ServersV263Test):
+    """Tests for creating a server with a block_device_mapping_v2 entry
+    using volume_type for microversion 2.67.
+    """
+    api_version = '2.67'
+
+    def test_create_server_boot_from_volume_with_volume_type(self):
+        bdm = [{"volume_size": 1,
+                "uuid": "11111111-1111-1111-1111-111111111111",
+                "delete_on_termination": True,
+                "source_type": "snapshot",
+                "destination_type": "volume",
+                "boot_index": 0,
+                "volume_type": "rbd"}]
+        s = self.cs.servers.create(
+            name="bfv server", image='', flavor=1,
+            nics='auto', block_device_mapping_v2=bdm)
+        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers', {
+            'server': {
+                'flavorRef': '1',
+                'imageRef': '',
+                'name': 'bfv server',
+                'networks': 'auto',
+                'block_device_mapping_v2': bdm,
+                'min_count': 1,
+                'max_count': 1,
+            }})
+
+    def test_create_server_boot_from_volume_with_volume_type_pre_267(self):
+        self.cs.api_version = api_versions.APIVersion('2.66')
+        bdm = [{"volume_size": 1,
+                "uuid": "11111111-1111-1111-1111-111111111111",
+                "delete_on_termination": True,
+                "source_type": "snapshot",
+                "destination_type": "volume",
+                "boot_index": 0,
+                "volume_type": "rbd"}]
+        ex = self.assertRaises(ValueError, self.cs.servers.create,
+                               name="bfv server", image='', flavor=1,
+                               nics='none', block_device_mapping_v2=bdm)
+        self.assertIn("Block device volume_type is not supported before "
+                      "microversion 2.67", six.text_type(ex))
