@@ -29,6 +29,7 @@ from novaclient.v2 import flavors
 from novaclient.v2 import hypervisors
 from novaclient.v2 import images
 from novaclient.v2 import instance_action
+from novaclient.v2 import instance_usage_audit_log
 from novaclient.v2 import keypairs
 from novaclient.v2 import limits
 from novaclient.v2 import list_extensions
@@ -169,6 +170,8 @@ class Client(object):
             assisted_volume_snapshots.AssistedSnapshotManager(self)
         self.cells = cells.CellsManager(self)
         self.instance_action = instance_action.InstanceActionManager(self)
+        self.instance_usage_audit_log = \
+            instance_usage_audit_log.InstanceUsageAuditLogManager(self)
         self.list_extensions = list_extensions.ListExtManager(self)
         self.migrations = migrations.MigrationManager(self)
         self.server_external_events = \
@@ -203,7 +206,7 @@ class Client(object):
             endpoint_type=endpoint_type,
             http_log_debug=http_log_debug,
             insecure=insecure,
-            logger=self.logger,
+            logger=logger,
             os_cache=self.os_cache,
             password=password,
             project_domain_id=project_domain_id,
